window.Gleam.initSearch([{"doc":"arctic","title":"arctic","content":"# Arctic\n\n[![Package Version](https://img.shields.io/hexpm/v/arctic)](https://hex.pm/packages/arctic)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/arctic/)\n\nArctic is a declarative web framework for building fast sites in the way you want. You specify what sorts of elements a markup file can have and how you notate those elements, and Arctic cranks out an optimized site that can be hosted on a static CDN and supported by servers or serverless functions. This maximizes page startup times.\n\nArctic doesn&#39;t take away your range of expression, it merely tries to act as shorthand. To do so, Arctic uses a modular process that lets you describe each part of each page in the way you want. A page can be mostly written in customized markup, and then in the middle you can write latex, HTML, gleam, or your own thing!\n\nArctic is still very much a work in progress. Packages for common languages like those mentioned above should be added. We need better comments and documentation. Pages aren&#39;t well optimized yet. The rendering part is still pretty DIY. If you&#39;re interested in this project, please chip in! Anyone is welcome to contribute. Also consider sponsoring me, so I have more time to work on this!\n\nArctic is very frontend-oriented. I think it would be paired very well with the sort of work coming out of the Pevensie project, which is generally on the server side. In the more immediate future, Wisp/Mist is a great Gleam backend that&#39;s available right now! More generally, you should be able to pair an Arctic codebase with servers written in any language, or serverless functions, or just serve the Arctic site statically from a CDN!\n\n```sh\ngleam add arctic\n```\n```gleam\nimport arctic/build\nimport arctic/collection\nimport arctic/config\nimport arctic/parse\nimport app\nimport lustre/element/html\nimport snag\n\npub fn main() {\n  let post_parser = parse.new()\n    |&gt; parse.add_inline_rule(&quot;_&quot;, &quot;_&quot;, parse.wrap(html.i))\n  let posts = collection.new(&quot;posts&quot;)\n    |&gt; collection.with_parser(post_parser)\n    |&gt; collection.with_index(app.post_index)\n    |&gt; collection.with_renderer(app.post_renderer)\n  let config = config.new()\n    |&gt; config.home_renderer(app.render_homepage)\n    |&gt; config.add_collection(posts)\n    |&gt; config.add_main_page(&quot;404&quot;, app.unknown_page_html)\n  let res = build.build(config)\n  case res {\n    Ok(Nil) -&gt; io.println(&quot;Success!&quot;)\n    Error(err) -&gt; io.println(snag.pretty_print(err))\n  }\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/arctic&gt;.\n\n## Development\nSome features mentioned here can be separate libraries on Hex, called `arctic_rss` or whatever and allowing imports like `arctic/rss`. That&#39;d be cool. Maybe `arctic_plugin_` is a better prefix, with `arctic/plugin/rss` for example, to ensure safer namespacing. Obviously anyone could develop these packages, and a bigger ecosystem is better!\n\nHere&#39;s a list of things I want to accomplish. They generally aren&#39;t hard. For some of the components I imagined just `shellout`ing to other tools and `simplifile`ing the result back into memory, or into an assets folder, or whatever. I already do this with latex on my personal site. However, the list is long, and contributions from others would help a lot in getting it done. Note that there are other important things Arctic needs to be able to do; me not including them here just means that Arctic is already doing them :)\n\n(Note from the future: As I complete these, I mark them instead of deleting them, just for fun. So some of these are already done!)\n\n - [x] Threaded state of page parsing, such as for generating unique IDs and filenames.\n - [x] Caching of whole processed pages when they haven&#39;t changed.\n - [ ] Caching of the output of components when we know that they haven&#39;t changed since last process. We can optimize a lot by only building the paragraphs that have changed, at the cost of hard-disk space. See comments on memory usage below. Assets generated by components, like images or whatever, wouldn&#39;t get rerendered if the component isn&#39;t rerun, so they get that caching for free. Notice that with Lustre Dev Tools watching for file changes, every rebuild would instantly put the rendered stuff on the page in real time. If we wanted to go real crazy we could file-watch the arctic markup files too, so you could edit in your DSL and watch the results appear in real time!\n - [x] Right now pages are just thrown in a directory and served statically. We could instead generate a collection of SSR&#39;d SPA `index.html` files, that route with `modem` and query for pages from a separate static collection, injecting the HTML a little like HTMX. This would be part of the optimization Arctic hopes to offer. It&#39;s reminiscent of the way AstroJS works, from what I understand. We could always offer a `Config` setting, or a different `build` function, to use the normal process instead. Note that we&#39;ll need to restructure a little for this, because the stuff that an SPA keeps around (nav bar, side panels, footer, etc.) isn&#39;t represented in any usable way, it&#39;s just produced by render functions. We can use the global event bus to tell them to rerender and how, if they need to. The SPA approach shouldn&#39;t be less expressive.\n - [ ] There isn&#39;t a great way to use a whole-page Lustre application that is partially written with Arctic markup. We should be able to inject Arctic-rendered HTML into normal gleam applications, to be used in, say, a Lustre `view` function.\n - [ ] It would be good to integrate with `nakai`, due to its use in the community, but I&#39;m really not sure how that would work. I need to think about it more.\n - [ ] A `sketch` integration would be cool. I&#39;ve talked to the author about it but that&#39;s all so far. I&#39;m not sure exactly what we&#39;d do differently from `sketch/lustre`, maybe just routing the styles in an optimal way into parts of the produced site.\n - [ ] Main pages are always rendered as a directory with an `index.html` file right now. It&#39;d be good to change that so you can do that if you want or you can add something like a `404.html`.\n - [ ] On the BEAM target, we should be able to trivially parallelize a ton of Arctic&#39;s work.\n - [ ] Generate a `sitemap.xml` for improved performance and SEO.\n - [ ] Better rendering convenience/modularity. It should be like how parsing is now.\n - [ ] Better metadata validation.\n - [ ] Better Lustre component (custom element) support. They&#39;re a big pain to use right now and it&#39;d be great if you could write the model type and the three functions as a block in the page and have that get copy-pasted into a gleam file to produce the custom component. This is super doable and can give correct error message file positions. Ideally this can even import from your other gleam files, we just have to make sure the file paths line up the right way.\n - [ ] RSS feed renderer\n - [ ] HTML component\n - [ ] Common inline rules: bold, links, strikethrough, underline, code, latex, footnote, etc.\n - [ ] Common prefix rules: lists, headers, horizontal rule, image, code block with language, footnote, table, block quote, etc.\n - [ ] A better default parser, with all the expected markdown features.\n - [ ] Latex component (builds on your own system with your packages)\n - [ ] Gleam component (imports and correct error message positions should be possible, with some pre- and post-processing)\n\nSomething that is a pain point for me is scale: handing around the entirety of all pages in all collections, in memory via pointer-chasing, could never scale to the number of pages of a large corporation. That limits one&#39;s growth when using Arctic. I think a streaming version (as well as one that doesn&#39;t need the posts to be sitting in a directory on each developer&#39;s machine!) can be implemented, but the API would need to change a bit to enable that. Servers searching through pages should also not have to re-process the markup files; we could build a database during processing. Ideally much of this should be as incremental as possible, which can be done well with caching. There&#39;s much figuring out to do.\n","url":"index.html"},{"doc":"arctic","title":"CacheablePage","content":"pub type CacheablePage {\n  CachedPage(path: String, metadata: Dict(String, String))\n  NewPage(Page)\n}\n\nCachedPage(path: String, metadata: Dict(String, String))\n\n\nNewPage(Page)\n\n\nSynonyms:\narctic.CacheablePage\narctic CacheablePage","url":"arctic.html#CacheablePage"},{"doc":"arctic","title":"Collection","content":"pub type Collection {\n  Collection(\n    directory: String,\n    parse: fn(String, String) -&gt; Result(Page),\n    index: Option(fn(List(CacheablePage)) -&gt; Element(Nil)),\n    feed: Option(#(String, fn(List(CacheablePage)) -&gt; String)),\n    ordering: fn(Page, Page) -&gt; Order,\n    render: fn(Page) -&gt; Element(Nil),\n    raw_pages: List(RawPage),\n  )\n}\n A particular organization of pages, \n written either in custom markup or raw HTML.\n A collection might be a set of products, or blog posts, or wiki entries.\n A dedicated page can be generated to show off the pages in a collection,\n and RSS feeds and whatnot can also be generated from the set of pages.\n The pages are pulled from a particular directory,\n and parsed and rendered a particular way,\n that might be different from other collections.\n See `arctic/collection` for more.\nCollection(\n  directory: String,\n  parse: fn(String, String) -&gt; Result(Page),\n  index: Option(fn(List(CacheablePage)) -&gt; Element(Nil)),\n  feed: Option(#(String, fn(List(CacheablePage)) -&gt; String)),\n  ordering: fn(Page, Page) -&gt; Order,\n  render: fn(Page) -&gt; Element(Nil),\n  raw_pages: List(RawPage),\n)\n\n\nSynonyms:\narctic.Collection\narctic Collection","url":"arctic.html#Collection"},{"doc":"arctic","title":"Config","content":"pub type Config {\n  Config(\n    render_home: fn(List(ProcessedCollection)) -&gt; Element(Nil),\n    main_pages: List(RawPage),\n    collections: List(Collection),\n    render_spa: Option(fn(Element(Nil)) -&gt; Element(Nil)),\n  )\n}\n An Arctic configuration, describing all the collections, pages, parsing rules, etc.\nConfig(\n  render_home: fn(List(ProcessedCollection)) -&gt; Element(Nil),\n  main_pages: List(RawPage),\n  collections: List(Collection),\n  render_spa: Option(fn(Element(Nil)) -&gt; Element(Nil)),\n)\n\n\nSynonyms:\narctic.Config\narctic Config","url":"arctic.html#Config"},{"doc":"arctic","title":"Page","content":"pub type Page {\n  Page(\n    id: String,\n    body: List(Element(Nil)),\n    metadata: Dict(String, String),\n    title: String,\n    blerb: String,\n    tags: List(String),\n    date: Option(Time),\n  )\n}\n A single page in a collection.\n These must have an ID to distinguish from other pages in the collection,\n and a body of HTML elements.\n Any other metadata can be added (stringly typed), \n and there are a variety of privileged metadata fields\n like `.title` and `.date` that are actual typed properties.\n However, these fields are optional.\n See `arctic/page` for more.\nPage(\n  id: String,\n  body: List(Element(Nil)),\n  metadata: Dict(String, String),\n  title: String,\n  blerb: String,\n  tags: List(String),\n  date: Option(Time),\n)\n\n\nSynonyms:\narctic.Page\narctic Page","url":"arctic.html#Page"},{"doc":"arctic","title":"ProcessedCollection","content":"pub type ProcessedCollection {\n  ProcessedCollection(\n    collection: Collection,\n    pages: List(CacheablePage),\n  )\n}\n A collection whose pages have been processed from the files.\nProcessedCollection(\n  collection: Collection,\n  pages: List(CacheablePage),\n)\n\n\nSynonyms:\narctic.ProcessedCollection\narctic ProcessedCollection","url":"arctic.html#ProcessedCollection"},{"doc":"arctic","title":"RawPage","content":"pub type RawPage {\n  RawPage(id: String, html: Element(Nil))\n}\n A page that is just HTML produced by hand.\nRawPage(id: String, html: Element(Nil))\n\n\nSynonyms:\narctic.RawPage\narctic RawPage","url":"arctic.html#RawPage"},{"doc":"arctic","title":"get_id","content":"pub fn get_id(p: CacheablePage) -&gt; String\n\nSynonyms:\narctic.get_id\narctic get_id","url":"arctic.html#get_id"},{"doc":"arctic","title":"output_path","content":"pub fn output_path(input_path: String) -&gt; String\n\nSynonyms:\narctic.output_path\narctic output_path","url":"arctic.html#output_path"},{"doc":"arctic","title":"to_dummy_page","content":"pub fn to_dummy_page(c: CacheablePage) -&gt; Page\n\nSynonyms:\narctic.to_dummy_page\narctic to_dummy_page","url":"arctic.html#to_dummy_page"},{"doc":"arctic","title":"arctic","content":"","url":"arctic.html"},{"doc":"arctic/build","title":"build","content":"pub fn build(config: Config) -&gt; Result(Nil, Snag)\n Fill out an `arctic_build` directory from an Arctic configuration.\nSynonyms:\narctic/build.build\narctic/build build","url":"arctic/build.html#build"},{"doc":"arctic/build","title":"arctic/build","content":"","url":"arctic/build.html"},{"doc":"arctic/page","title":"new","content":"pub fn new(id: String) -&gt; Page\n Construct a new page, with the given ID and default-everything.\nSynonyms:\narctic/page.new\narctic/page new","url":"arctic/page.html#new"},{"doc":"arctic/page","title":"replace_metadata","content":"pub fn replace_metadata(\n  p: Page,\n  metadata: Dict(String, String),\n) -&gt; Page\n Swap out the entirety of the metadata for a page with a new dictionary,\n except for the privileged metadata like `.title` and `.date`.\n This is useful for if you&#39;re building a metadata dictionary programmatically.\nSynonyms:\narctic/page.replace_metadata\narctic/page replace_metadata","url":"arctic/page.html#replace_metadata"},{"doc":"arctic/page","title":"with_blerb","content":"pub fn with_blerb(p: Page, blerb: String) -&gt; Page\n Add a blerb (description, whatever) to a page.\n This is useful for nice thumbnails.\nSynonyms:\narctic/page.with_blerb\narctic/page with_blerb","url":"arctic/page.html#with_blerb"},{"doc":"arctic/page","title":"with_body","content":"pub fn with_body(p: Page, body: List(Element(Nil))) -&gt; Page\n Add a &quot;body&quot; to a page. \n A body is the list of elements that will appear when the page is loaded.\nSynonyms:\narctic/page.with_body\narctic/page with_body","url":"arctic/page.html#with_body"},{"doc":"arctic/page","title":"with_date","content":"pub fn with_date(p: Page, date: Time) -&gt; Page\n Add a date to a page.\n This is useful for sorting pages by date in a list,\n like in a blog.\nSynonyms:\narctic/page.with_date\narctic/page with_date","url":"arctic/page.html#with_date"},{"doc":"arctic/page","title":"with_metadata","content":"pub fn with_metadata(p: Page, key: String, val: String) -&gt; Page\n Add some metadata to a page.\n This is any string key and value, that you can look up during parsing later.\n Sorry for the lack of type safety! \n Processing mismatches are handled with `snag` results,\n which is like compile-time errors since the run-time is at build-time.\n Also, note that some metadata gets privileged fields store in a different way, \n like `.date`. This adds type safety and convenience, and is opt-in.\nSynonyms:\narctic/page.with_metadata\narctic/page with_metadata","url":"arctic/page.html#with_metadata"},{"doc":"arctic/page","title":"with_tags","content":"pub fn with_tags(p: Page, tags: List(String)) -&gt; Page\n Add tags to a page.\n This is useful for implementing a helpful search.\nSynonyms:\narctic/page.with_tags\narctic/page with_tags","url":"arctic/page.html#with_tags"},{"doc":"arctic/page","title":"with_title","content":"pub fn with_title(p: Page, title: String) -&gt; Page\n Add a title to a page.\nSynonyms:\narctic/page.with_title\narctic/page with_title","url":"arctic/page.html#with_title"},{"doc":"arctic/page","title":"arctic/page","content":"","url":"arctic/page.html"},{"doc":"arctic/parse","title":"ParseData","content":"pub opaque type ParseData(a)\n The data accessible while parsing, such as current position or filename.\n\nSynonyms:\narctic/parse.ParseData\narctic/parse ParseData","url":"arctic/parse.html#ParseData"},{"doc":"arctic/parse","title":"ParserBuilder","content":"pub opaque type ParserBuilder(a)\n An under-construction parser that you can add rules to.\n For example,\n \n my_parser\n   |&gt; add_inline_rule(&quot;_&quot;, &quot;_&quot;, wrap_inline(html.i))\n \n\nSynonyms:\narctic/parse.ParserBuilder\narctic/parse ParserBuilder","url":"arctic/parse.html#ParserBuilder"},{"doc":"arctic/parse","title":"Position","content":"pub type Position {\n  Position(line: Int, column: Int)\n}\n A place in an Arctic markup file\nPosition(line: Int, column: Int)\n\n\nSynonyms:\narctic/parse.Position\narctic/parse Position","url":"arctic/parse.html#Position"},{"doc":"arctic/parse","title":"add_dynamic_component","content":"pub fn add_dynamic_component(\n  p: ParserBuilder(a),\n  name: String,\n) -&gt; ParserBuilder(a)\n Add a &quot;dynamic component&quot; to a parser.\n A dynamic component is a component (imagine a DSL) that needs MVC interactivity.\n You will need to separately register a Lustre component of the same name;\n this is just the way that you put it into your site from an Arctic markup file.\n In your Arctic markup file, you would write\n \n @component_name(an arg, another arg)\n A bunch\n of content\n \n Arctic will parse the body until the first blank line.\n Then the produced HTML is\n \n &lt;component_name data-parameters=&quot;an arg,another arg&quot; data-body=&quot;A bunch\\nof content&quot;&gt;\n &lt;/component_name&gt;\n \nSynonyms:\narctic/parse.add_dynamic_component\narctic/parse add_dynamic_component","url":"arctic/parse.html#add_dynamic_component"},{"doc":"arctic/parse","title":"add_inline_rule","content":"pub fn add_inline_rule(\n  p: ParserBuilder(a),\n  left: String,\n  right: String,\n  action: fn(Element(Nil), List(String), ParseData(a)) -&gt;\n    Result(#(Element(Nil), a), Snag),\n) -&gt; ParserBuilder(a)\n Add an &quot;inline rule&quot; to a parser.\n An inline rule rewrites parts of text paragraphs.\n For example, `add_inline_rule(&quot;**&quot;, &quot;**&quot;, wrap_inline(html.strong))` \n replaces anything wrapped in double-asterisks with a bolded version of the same text.\n Note that the rule may fail with a `snag` error, halting the parsing of that paragraph,\n and that the position in the file is given, so you can produce better `snag` error messages.\n The rewrite might also be given parameters, allowing for something like\n `[here](https://example.com) is a link`\nSynonyms:\narctic/parse.add_inline_rule\narctic/parse add_inline_rule","url":"arctic/parse.html#add_inline_rule"},{"doc":"arctic/parse","title":"add_prefix_rule","content":"pub fn add_prefix_rule(\n  p: ParserBuilder(a),\n  prefix: String,\n  action: fn(Element(Nil), ParseData(a)) -&gt;\n    Result(#(Element(Nil), a), Snag),\n) -&gt; ParserBuilder(a)\n Add a &quot;prefix rule&quot; to a parser.\n A prefix rule rewrites a whole paragraph based on symbols at the beginning.\n For example, `add_prefix_rule(&quot;#&quot;, wrap_prefix(html.h1))` \n replaces anything that starts with a hashtag with a header of the same text.\n Note that the rule may fail with a `snag` error, halting the parsing of that paragraph,\n and that the position in the file is given, so you can produce better `snag` error messages.\nSynonyms:\narctic/parse.add_prefix_rule\narctic/parse add_prefix_rule","url":"arctic/parse.html#add_prefix_rule"},{"doc":"arctic/parse","title":"add_static_component","content":"pub fn add_static_component(\n  p: ParserBuilder(a),\n  name: String,\n  action: fn(List(String), String, ParseData(a)) -&gt;\n    Result(#(Element(Nil), a), Snag),\n) -&gt; ParserBuilder(a)\n Add a &quot;static component&quot; to a parser.\n A static component is a component (imagine a DSL) that doesn&#39;t need MVC interactivity.\n You just specify how it gets turned into HTML,\n and Arctic turns it into HTML.\n In your Arctic markup file, you write \n \n @component_name(an arg, another arg)\n A bunch\n of content\n \n Arctic will parse the body until the first blank line, then apply your given action to the parameters and body.\n This allows you to embed languages in Arctic markup files, like latex or HTML.\n Note that the component may fail with a `snag` error, halting the parsing of that paragraph,\n and that the position in the file is given, so you can produce better `snag` error messages.\nSynonyms:\narctic/parse.add_static_component\narctic/parse add_static_component","url":"arctic/parse.html#add_static_component"},{"doc":"arctic/parse","title":"get_metadata","content":"pub fn get_metadata(data: ParseData(a)) -&gt; Dict(String, String)\n\nSynonyms:\narctic/parse.get_metadata\narctic/parse get_metadata","url":"arctic/parse.html#get_metadata"},{"doc":"arctic/parse","title":"get_pos","content":"pub fn get_pos(data: ParseData(a)) -&gt; Position\n\nSynonyms:\narctic/parse.get_pos\narctic/parse get_pos","url":"arctic/parse.html#get_pos"},{"doc":"arctic/parse","title":"get_state","content":"pub fn get_state(data: ParseData(a)) -&gt; a\n\nSynonyms:\narctic/parse.get_state\narctic/parse get_state","url":"arctic/parse.html#get_state"},{"doc":"arctic/parse","title":"new","content":"pub fn new(start_state: a) -&gt; ParserBuilder(a)\n Create a new parser builder, with no rules or components.\n It only has an initial state.\nSynonyms:\narctic/parse.new\narctic/parse new","url":"arctic/parse.html#new"},{"doc":"arctic/parse","title":"parse","content":"pub fn parse(\n  p: ParserBuilder(a),\n  src_name: String,\n  src: String,\n) -&gt; Result(Page, Snag)\n Apply a given parser to a given string.\nSynonyms:\narctic/parse.parse\narctic/parse parse","url":"arctic/parse.html#parse"},{"doc":"arctic/parse","title":"with_state","content":"pub fn with_state(data: ParseData(a), state: a) -&gt; ParseData(a)\n\nSynonyms:\narctic/parse.with_state\narctic/parse with_state","url":"arctic/parse.html#with_state"},{"doc":"arctic/parse","title":"wrap_inline","content":"pub fn wrap_inline(\n  w: fn(List(Attribute(a)), List(Element(Nil))) -&gt; Element(Nil),\n) -&gt; fn(Element(Nil), b, ParseData(c)) -&gt;\n  Result(#(Element(Nil), c), d)\n A convenience function for inline rules that just put content in an element.\n For example, `wrap_inline(html.i)` italicizes.\nSynonyms:\narctic/parse.wrap_inline\narctic/parse wrap_inline","url":"arctic/parse.html#wrap_inline"},{"doc":"arctic/parse","title":"wrap_inline_with_attributes","content":"pub fn wrap_inline_with_attributes(\n  w: fn(List(Attribute(a)), List(Element(Nil))) -&gt; Element(Nil),\n  attrs: List(Attribute(a)),\n) -&gt; fn(Element(Nil), b, ParseData(c)) -&gt;\n  Result(#(Element(Nil), c), d)\n A convenience function for inline rules that just put content in an element \n and give the element some parameters.\n For example, `wrap_inline(html.a, [attribute.src(&quot;https://arctic-framework.org&quot;)])`\n makes something a link to arctic-framework.org.\nSynonyms:\narctic/parse.wrap_inline_with_attributes\narctic/parse wrap_inline_with_attributes","url":"arctic/parse.html#wrap_inline_with_attributes"},{"doc":"arctic/parse","title":"wrap_prefix","content":"pub fn wrap_prefix(\n  w: fn(List(Attribute(a)), List(Element(Nil))) -&gt; Element(Nil),\n) -&gt; fn(Element(Nil), ParseData(b)) -&gt;\n  Result(#(Element(Nil), b), c)\n A convenience function for prefix rules that just put content in an element\n For example, `wrap_prefix(html.h1)` makes a paragraph a header.\nSynonyms:\narctic/parse.wrap_prefix\narctic/parse wrap_prefix","url":"arctic/parse.html#wrap_prefix"},{"doc":"arctic/parse","title":"wrap_prefix_with_attributes","content":"pub fn wrap_prefix_with_attributes(\n  w: fn(List(Attribute(a)), List(Element(Nil))) -&gt; Element(Nil),\n  attrs: List(Attribute(a)),\n) -&gt; fn(Element(Nil), ParseData(b)) -&gt;\n  Result(#(Element(Nil), b), c)\n A convenience function for prefix rules that just put content in an element \n and give the element some parameters.\n For example, `wrap_prefix(html.a, [attribute.src(&quot;https://arctic-framework.org&quot;)])`\n makes a paragraph a link to arctic-framework.org.\nSynonyms:\narctic/parse.wrap_prefix_with_attributes\narctic/parse wrap_prefix_with_attributes","url":"arctic/parse.html#wrap_prefix_with_attributes"},{"doc":"arctic/parse","title":"arctic/parse","content":"","url":"arctic/parse.html"},{"doc":"arctic/collection","title":"default_parser","content":"pub fn default_parser() -&gt; fn(String, String) -&gt;\n  Result(Page, Snag)\n A simple default parser for the sorts of things you&#39;d expect when writing markup.\n This also serves as a nice example of how to construct parsers.\nSynonyms:\narctic/collection.default_parser\narctic/collection default_parser","url":"arctic/collection.html#default_parser"},{"doc":"arctic/collection","title":"new","content":"pub fn new(dir: String) -&gt; Collection\n Produce a new collection, with default-everything and the given directory path.\n You can use the other functions to modify the collection.\n Or, collections can be produced manually with the `Collection` constructor from `arctic`.\nSynonyms:\narctic/collection.new\narctic/collection new","url":"arctic/collection.html#new"},{"doc":"arctic/collection","title":"with_feed","content":"pub fn with_feed(\n  c: Collection,\n  filename: String,\n  render: fn(List(CacheablePage)) -&gt; String,\n) -&gt; Collection\n Add a &quot;feed&quot; to the collection.\n A feed is a special text file generated based on the elements of the collection.\n An RSS feed would be done this way.\nSynonyms:\narctic/collection.with_feed\narctic/collection with_feed","url":"arctic/collection.html#with_feed"},{"doc":"arctic/collection","title":"with_index","content":"pub fn with_index(\n  c: Collection,\n  index: fn(List(CacheablePage)) -&gt; Element(Nil),\n) -&gt; Collection\n Add an &quot;index&quot; to the collection.\n An index is a page that shows off the pages in the collection, \n perhaps with a search bar and/or a list of pretty thumbnails.\n Note that this would need to bring *all* the pages to the client side;\n Pagination and search-via-server should be done in other ways.\n Though this doesn&#39;t scale well to massive numbers of pages,\n it&#39;s pretty easy to swap it out with something else when the number gets too high.\nSynonyms:\narctic/collection.with_index\narctic/collection with_index","url":"arctic/collection.html#with_index"},{"doc":"arctic/collection","title":"with_ordering","content":"pub fn with_ordering(\n  c: Collection,\n  ordering: fn(Page, Page) -&gt; Order,\n) -&gt; Collection\n Add an ordering to a collection.\n This specifies the order in which pages are listed \n on, say, a collection index.\nSynonyms:\narctic/collection.with_ordering\narctic/collection with_ordering","url":"arctic/collection.html#with_ordering"},{"doc":"arctic/collection","title":"with_parser","content":"pub fn with_parser(\n  c: Collection,\n  parser: fn(String, String) -&gt; Result(Page, Snag),\n) -&gt; Collection\n Add a parser to a collection.\n A parser processed the raw text and \n either fails with a message or produces a page.\n See `arctic/parse` for help constructing these.\nSynonyms:\narctic/collection.with_parser\narctic/collection with_parser","url":"arctic/collection.html#with_parser"},{"doc":"arctic/collection","title":"with_raw_page","content":"pub fn with_raw_page(\n  c: Collection,\n  id: String,\n  body: Element(Nil),\n) -&gt; Collection\n Add a &quot;raw page&quot; to a collection.\n A raw page is just HTML, \n no parsing or processing will get applied.\nSynonyms:\narctic/collection.with_raw_page\narctic/collection with_raw_page","url":"arctic/collection.html#with_raw_page"},{"doc":"arctic/collection","title":"with_renderer","content":"pub fn with_renderer(\n  c: Collection,\n  renderer: fn(Page) -&gt; Element(Nil),\n) -&gt; Collection\n Add a &quot;renderer&quot; to a collection.\n A renderer is any Page-&gt;HTML function.\nSynonyms:\narctic/collection.with_renderer\narctic/collection with_renderer","url":"arctic/collection.html#with_renderer"},{"doc":"arctic/collection","title":"arctic/collection","content":"","url":"arctic/collection.html"},{"doc":"arctic/config","title":"add_collection","content":"pub fn add_collection(\n  config: Config,\n  collection: Collection,\n) -&gt; Config\n Add a &quot;collection&quot; to an Arctic configuration.\n A collection holds a bunch of pages and their processing rules,\n like a set of products, blog posts, wiki entries, etc.\n See `arctic/collection` for more.\nSynonyms:\narctic/config.add_collection\narctic/config add_collection","url":"arctic/config.html#add_collection"},{"doc":"arctic/config","title":"add_main_page","content":"pub fn add_main_page(\n  config: Config,\n  id: String,\n  body: Element(Nil),\n) -&gt; Config\n Add a &quot;main page&quot; to an Arctic configuration.\n Main pages are pages that aren&#39;t a part of any collection, like &quot;Contact&quot; or &quot;About.&quot;\n Note that the home page (`/index.html`) is handled via `home_renderer` instead.\nSynonyms:\narctic/config.add_main_page\narctic/config add_main_page","url":"arctic/config.html#add_main_page"},{"doc":"arctic/config","title":"add_spa_frame","content":"pub fn add_spa_frame(\n  config: Config,\n  frame: fn(Element(Nil)) -&gt; Element(Nil),\n) -&gt; Config\n Specify code that is on the outside of a page, \n and doesn&#39;t get re-rendered on page navigation.\n This can be nav bars, a `head` element, side panels, footer, etc.\nSynonyms:\narctic/config.add_spa_frame\narctic/config add_spa_frame","url":"arctic/config.html#add_spa_frame"},{"doc":"arctic/config","title":"home_renderer","content":"pub fn home_renderer(\n  config: Config,\n  renderer: fn(List(ProcessedCollection)) -&gt; Element(Nil),\n) -&gt; Config\n Set the renderer for the home page of a site (`/index.html`).\n Note that a list of all collections, with all of their pages, is provided if you&#39;d like to use it.\nSynonyms:\narctic/config.home_renderer\narctic/config home_renderer","url":"arctic/config.html#home_renderer"},{"doc":"arctic/config","title":"new","content":"pub fn new() -&gt; Config\n Produce a new Arctic configuration, with default settings.\n An Arctic configuration holds all the collections, pages, parsing rules, etc.\nSynonyms:\narctic/config.new\narctic/config new","url":"arctic/config.html#new"},{"doc":"arctic/config","title":"turn_off_spa","content":"pub fn turn_off_spa(config: Config) -&gt; Config\n Generate the site as a directory of files,\n instead of as a single-page app with clever routing.\nSynonyms:\narctic/config.turn_off_spa\narctic/config turn_off_spa","url":"arctic/config.html#turn_off_spa"},{"doc":"arctic/config","title":"arctic/config","content":"","url":"arctic/config.html"}]);